import java.awt.*;
import java.util.Vector;
import dao_dbengine;
import dao3032.*;
import com.ms.com.ComException;
import StatusPanel;
import UserInformation;
import ChoiceData;
import ListData;

/*
 *
 * AdministrationPanel
 *
 */
public class AdministrationPanel extends StatusPanel
{
	protected Database db;
	protected UserInformation UserInfo;

	public AdministrationPanel(Database db, UserInformation UserInfo)
	{
		super();

		// save the passed in information
		this.db = db;
		this.UserInfo = UserInfo;

		// make a grid bag layout for the panel
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		setLayout(layout);

		gbc.weightx = 1.0;				// let list fill remaining space
		gbc.weighty = 1.0;				// let list fill remaining space
		gbc.gridwidth = 1;
		gbc.gridheight = 2;
		gbc.fill = GridBagConstraints.BOTH;
		OperatorPanel opPanel = new OperatorPanel(db);
		layout.setConstraints(opPanel, gbc);
		add(opPanel);

		gbc.gridheight = 1;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		DepartmentPanel deptPanel = new DepartmentPanel(db);
		layout.setConstraints(deptPanel, gbc);
		add(deptPanel);

		gbc.weightx = 0.0;
		gbc.weighty = 0.0;
		gbc.anchor = GridBagConstraints.EAST;
		gbc.fill = GridBagConstraints.NONE;
		Button btn = new Button("Done");
		layout.setConstraints(btn, gbc);
		add(btn);
	}

	public boolean action(Event evt, Object obj)
	{
		boolean retval = false;			// assume not processed

		if ("Done".equals(obj))
		{
			UserInfo.signalNextScreen();

			retval = true;
		}

		return retval;
	}
}

class OperatorPanel extends StatusPanel
{
	protected Database db;
	protected TextField UserNameField = new TextField(8);
	protected TextField PasswordField = new TextField(8);
	protected TextField ConfirmPasswordField = new TextField(8);
	protected TextField FirstNameField = new TextField(30);
	protected TextField LastNameField = new TextField(30);
	protected ChoiceData AccessLevelChoice = new ChoiceData();
	protected ListData OperatorList = new ListData();
	protected Operator CurrentOperator = new Operator();

	public OperatorPanel(Database db)
	{
		super(true);

		// save the passed in information
		this.db = db;

		// make a grid bag layout for the panel
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		setLayout(layout);

		// put a little space above around the controls
		gbc.insets.left = gbc.insets.top = 
				gbc.insets.right = gbc.insets.bottom = 2;

		gbc.weighty = 1.0;				// let list fill remaining space
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.fill = GridBagConstraints.BOTH;
		layout.setConstraints(OperatorList, gbc);
		add(OperatorList);

		gbc.weighty = 0.0;				// don't resize remaining controls
		StatusPanel opInfo = getOperatorInfoPanel();
		layout.setConstraints(opInfo, gbc);
		add(opInfo);

		// add two side-by-side buttons
		gbc.weightx = 1.0;				// equally space horizontally
		gbc.gridwidth = 1;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		Button btn = new Button("New");
		layout.setConstraints(btn, gbc);
		add(btn);

		gbc.gridwidth = GridBagConstraints.REMAINDER;
		btn = new Button("Update");
		layout.setConstraints(btn, gbc);
		add(btn);

		// read in the operators from the database
		readOperators();
	}

	public boolean handleEvent(Event evt)
	{
		boolean retval;

		switch(evt.id) {
		case Event.LIST_SELECT:
		case Event.LIST_DESELECT:
			fillCurrentOperator();
			retval = true;
			break;
		default:
			retval = super.handleEvent(evt);
			break;
		}

		return retval;
	}

	public boolean action(Event evt, Object obj)
	{
		boolean retval = false;			// assume not processed

		if ("New".equals(obj))
		{
			if (validateFields())
			{
				try
				{
					CurrentOperator.insert(db);
					readOperators();
				}
				catch(ComException e)
				{
					setStatus(e);
				}
			}

			retval = true;
		}
		else if ("Update".equals(obj))
		{
			if (validateFields())
			{
				try
				{
					CurrentOperator.update(db);
					readOperators();
				}
				catch(Exception e)
				{
					setStatus("DB Error: " + e.getMessage());
				}
			}

			retval = true;
		}

		return retval;
	}

	// This method creates a panel for just the operator info.
	private StatusPanel getOperatorInfoPanel()
	{
		StatusPanel p = new StatusPanel();

		// make a grid bag layout for the panel
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		p.setLayout(layout);

		// put a little space above and below the controls
		gbc.insets.top = gbc.insets.bottom = 2;

		// create the controls for this panel
		gbc.weightx = 1.0;				// center 2 controls side-by-side
		gbc.anchor = GridBagConstraints.EAST;
		Label lbl = new Label("User Name:", Label.RIGHT);
		layout.setConstraints(lbl, gbc);
		p.add(lbl);

		// make the text field the last on the line
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		layout.setConstraints(UserNameField, gbc);
		p.add(UserNameField);

		gbc.anchor = GridBagConstraints.EAST;
		gbc.gridwidth = 1;
		lbl = new Label("Password:", Label.RIGHT);
		layout.setConstraints(lbl, gbc);
		p.add(lbl);

		// make the text field the last on the line
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		layout.setConstraints(PasswordField, gbc);
		PasswordField.setEchoCharacter('*');
		p.add(PasswordField);

		gbc.anchor = GridBagConstraints.EAST;
		gbc.gridwidth = 1;
		lbl = new Label("Confirm Password:", Label.RIGHT);
		layout.setConstraints(lbl, gbc);
		p.add(lbl);

		// make the text field the last on the line
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		layout.setConstraints(ConfirmPasswordField, gbc);
		ConfirmPasswordField.setEchoCharacter('*');
		p.add(ConfirmPasswordField);

		// remove the space, reset anchor, one control per line
		gbc.insets.top = gbc.insets.bottom = 0;
		gbc.anchor = GridBagConstraints.CENTER;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.fill = GridBagConstraints.HORIZONTAL;

		lbl = new Label("First Name:", Label.LEFT);
		layout.setConstraints(lbl, gbc);
		p.add(lbl);

		layout.setConstraints(FirstNameField, gbc);
		p.add(FirstNameField);

		lbl = new Label("Last Name:", Label.LEFT);
		layout.setConstraints(lbl, gbc);
		p.add(lbl);

		layout.setConstraints(LastNameField, gbc);
		p.add(LastNameField);

		// add the choices before adding the control
		AccessLevelChoice.addItemData("Administrator", Operator.ADMINISTRATOR);
		AccessLevelChoice.addItemData("Executive", Operator.EXECUTIVE);
		AccessLevelChoice.addItemData("Manager", Operator.MANAGER);
		AccessLevelChoice.addItemData("Programmer", Operator.PROGRAMMER);
		AccessLevelChoice.addItemData("Guest", Operator.GUEST);

		gbc.insets.top = gbc.insets.bottom = 2;
		gbc.fill = GridBagConstraints.NONE;

		layout.setConstraints(AccessLevelChoice, gbc);
		p.add(AccessLevelChoice);

		return p;
	}

	private void readOperators()
	{
		// kill existing list
		OperatorList.clear();
		Vector operators = new Vector();

		Operator.readAll(db, operators);

		OperatorList.load(operators);
	}

	private void fillCurrentOperator()
	{
		try
		{
			Operator op = (Operator)OperatorList.getSelectedData();

			CurrentOperator.set(op);
		}
		catch (ArrayIndexOutOfBoundsException e)
		{
			CurrentOperator.clear();
		}

		UserNameField.setText(CurrentOperator.UserName);
		PasswordField.setText(CurrentOperator.Password);
		ConfirmPasswordField.setText(CurrentOperator.Password);
		FirstNameField.setText(CurrentOperator.FirstName);
		LastNameField.setText(CurrentOperator.LastName);

		AccessLevelChoice.selectByData(CurrentOperator.AccessLevel);
	}

	private boolean validateFields()
	{
		boolean retval = false;			// assume not enough info

		if ((UserNameField.getText().length() > 0) && 
				(PasswordField.getText().equals(ConfirmPasswordField.getText())))
		{
			// move information to the current operator
			CurrentOperator.set(UserNameField.getText(), 
					PasswordField.getText(), 
					FirstNameField.getText(), 
					LastNameField.getText(), 
					(String)AccessLevelChoice.getSelectedData());

			retval = true;
		}

		return retval;
	}
}

class DepartmentPanel extends StatusPanel
{
	protected Database db;
	protected TextField CodeField = new TextField(4);
	protected TextField NameField = new TextField(20);
	protected ListData DepartmentList = new ListData();
	protected Department CurrentDept = new Department();

	public DepartmentPanel(Database db)
	{
		super(true);

		// save the passed in information
		this.db = db;

		// make a grid bag layout for the panel
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		setLayout(layout);

		// put a little space around the controls
		gbc.insets.left = gbc.insets.top = 
				gbc.insets.right = gbc.insets.bottom = 2;

		gbc.weighty = 1.0;				// let list fill remaining space
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.fill = GridBagConstraints.BOTH;
		layout.setConstraints(DepartmentList, gbc);
		add(DepartmentList);

		gbc.weighty = 0.0;				// don't resize remaining controls
		StatusPanel deptInfo = getDeptInfoPanel();
		layout.setConstraints(deptInfo, gbc);
		add(deptInfo);

		// add two side-by-side buttons
		gbc.weightx = 1.0;				// equally space horizontally
		gbc.gridwidth = 1;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		Button btn = new Button("New");
		layout.setConstraints(btn, gbc);
		add(btn);

		gbc.gridwidth = GridBagConstraints.REMAINDER;
		btn = new Button("Update");
		layout.setConstraints(btn, gbc);
		add(btn);

		// read in the operators from the database
		readDepartments();
	}

	public boolean handleEvent(Event evt)
	{
		boolean retval;

		switch(evt.id) {
		case Event.LIST_SELECT:
		case Event.LIST_DESELECT:
			fillCurrentDept();
			retval = true;
			break;
		default:
			retval = super.handleEvent(evt);
			break;
		}

		return retval;
	}

	public boolean action(Event evt, Object obj)
	{
		boolean retval = false;			// assume not processed

		if ("New".equals(obj))
		{
			if (validateFields())
			{
				try
				{
					CurrentDept.insert(db);
					readDepartments();
				}
				catch(ComException e)
				{
					setStatus("DB Error: " + e.getMessage());
				}
			}

			retval = true;
		}
		else if ("Update".equals(obj))
		{
			if (validateFields())
			{
				try
				{
					CurrentDept.update(db);
					readDepartments();
				}
				catch(Exception e)
				{
					setStatus("DB Error: " + e.getMessage());
				}
			}

			retval = true;
		}

		return retval;
	}

	// This method creates a panel for just the department info.
	private StatusPanel getDeptInfoPanel()
	{
		StatusPanel p = new StatusPanel();

		// make a grid bag layout for the panel
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		p.setLayout(layout);

		// put a little space above and below the controls
		gbc.insets.top = gbc.insets.bottom = 2;

		// create the controls for this panel
		gbc.weightx = 1.0;				// center 2 controls side-by-side
		gbc.anchor = GridBagConstraints.EAST;
		Label lbl = new Label("Dept Code:", Label.RIGHT);
		layout.setConstraints(lbl, gbc);
		p.add(lbl);

		// make the text field the last on the line
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		layout.setConstraints(CodeField, gbc);
		p.add(CodeField);

		gbc.anchor = GridBagConstraints.EAST;
		gbc.gridwidth = 1;
		lbl = new Label("Dept Name:", Label.RIGHT);
		layout.setConstraints(lbl, gbc);
		p.add(lbl);

		// make the text field the last on the line
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		layout.setConstraints(NameField, gbc);
		p.add(NameField);

		return p;
	}

	private void readDepartments()
	{
		// kill existing list
		DepartmentList.clear();
		Vector departments = new Vector();

		Department.readAll(db, departments);

		DepartmentList.load(departments);
	}

	private void fillCurrentDept()
	{
		try
		{
			Department dept = (Department)DepartmentList.getSelectedData();

			CurrentDept.set(dept);
		}
		catch (ArrayIndexOutOfBoundsException e)
		{
			CurrentDept.clear();
		}

		CodeField.setText(CurrentDept.Code);
		NameField.setText(CurrentDept.Name);
	}

	private boolean validateFields()
	{
		boolean retval = false;			// assume not enough info

		if (CodeField.getText().length() > 0)
		{
			// move information to the current department
			CurrentDept.set(CodeField.getText(), NameField.getText());

			retval = true;
		}

		return retval;
	}
}