import java.awt.*;
import java.applet.*;
import java.util.*;
import dao_dbengine;
import dao3032.*;
import com.ms.com.ComException;
import UserInformation;
import ListData;

/*
 *
 * ControlPanel
 *
 */
public class ControlPanel extends StatusPanel
{
	protected Database db;
	protected UserInformation UserInfo;
	protected Label NameLabel = new Label();
	protected Button AdministrationButton = new Button("Administration");
	protected ListData RequestList = new ListData();
	protected RequestPanel ReqPanel;

	public ControlPanel(Database db, UserInformation UserInfo)
	{
		super();

		// save the passed in information
		this.db = db;
		this.UserInfo = UserInfo;

		// make a grid bag layout for the panel
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		setLayout(layout);

		// put a little space above and below the controls
		gbc.insets.top = gbc.insets.bottom = 2;

		// create the controls for this panel
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		AdministrationButton.hide();
		Panel p = new Panel();
		p.add(new Button("New"));
		p.add(new Button("Details"));
		p.add(AdministrationButton);
		layout.setConstraints(p, gbc);
		add(p);

		gbc.weightx = 1.0;				// equally space remaining
		gbc.weighty = 1.0;				// let list fill remaining space
		gbc.gridwidth = GridBagConstraints.RELATIVE;
		gbc.fill = GridBagConstraints.BOTH;
		layout.setConstraints(RequestList, gbc);
		add(RequestList);

		gbc.gridwidth = GridBagConstraints.REMAINDER;
		ReqPanel = new RequestPanel(this, db);
		layout.setConstraints(ReqPanel, gbc);
		add(ReqPanel);

		readRequests();
	}

	public boolean action(Event evt, Object obj)
	{
		boolean retval = false;

		if ("New".equals(obj))
		{
			ReqPanel.setRequest(new Request());

			retval = true;
		}
		else if ("Details".equals(obj))
		{
			// get the selected request
			Request req = (Request)RequestList.getSelectedData();

			// display the request if one is selected
			if (req != null)
				if (req.read(db))
					ReqPanel.setRequest(req);

			retval = true;
		}
		else if ("Administration".equals(obj))
		{
			UserInfo.signalNextScreen();

			retval = true;
		}

		return retval;
	}

	public void show()
	{
		if (UserInfo.AccessLevel <= UserInformation.ADMINISTRATOR)
			AdministrationButton.show();

		super.show();
	}

	public void readRequests()
	{
		// kill existing list
		RequestList.clear();
		Vector requests = new Vector();

		// query the database for all of the requests
		Request.readAll(db, requests);

		// load the list
		RequestList.load(requests);

        // clear the current panel
		ReqPanel.setRequest(new Request());
	}
}

class RequestPanel extends StatusPanel
{
	protected ControlPanel conPanel;
	protected Database db;
	protected TextField TrackingIdField = new TextField(8);
	protected TextField SubmissionDateField = new TextField(8);
	protected TextField CompletionDateField = new TextField(8);
	protected TextField TitleField = new TextField(30);
	protected TextArea DescriptionArea = new TextArea();
	protected ChoiceData RequestorChoice = new ChoiceData();
	protected ChoiceData StatusChoice = new ChoiceData();
	protected ChoiceData ApprovedBudgetChoice = new ChoiceData();
	protected Request CurrentRequest = new Request();

	public RequestPanel(ControlPanel conPanel, Database db)
	{
		super(true);

		// save the passed in information
		this.conPanel = conPanel;
		this.db = db;

		// make a grid bag layout for the panel
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		setLayout(layout);

		// put a little space around the controls
		gbc.insets.left = gbc.insets.top = 
				gbc.insets.right = gbc.insets.bottom = 2;

		// create the controls for this panel
		gbc.anchor = GridBagConstraints.EAST;
		gbc.gridwidth = 1;
		gbc.fill = GridBagConstraints.BOTH;
		Label lbl = new Label("Id:", Label.RIGHT);
		layout.setConstraints(lbl, gbc);
		add(lbl);

		TrackingIdField.setEditable(false);
		layout.setConstraints(TrackingIdField, gbc);
		add(TrackingIdField);

		lbl = new Label("Submitted:", Label.RIGHT);
		layout.setConstraints(lbl, gbc);
		add(lbl);

		gbc.gridwidth = GridBagConstraints.REMAINDER;
		SubmissionDateField.setEditable(false);
		layout.setConstraints(SubmissionDateField, gbc);
		add(SubmissionDateField);

		// remove the space, reset anchor, one control per line
		gbc.insets.top = gbc.insets.bottom = 0;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.fill = GridBagConstraints.HORIZONTAL;

		lbl = new Label("Title:", Label.LEFT);
		layout.setConstraints(lbl, gbc);
		add(lbl);

		layout.setConstraints(TitleField, gbc);
		add(TitleField);

		lbl = new Label("Description:", Label.LEFT);
		layout.setConstraints(lbl, gbc);
		add(lbl);

		gbc.weighty = 1.0;				// absorb any space
		layout.setConstraints(DescriptionArea, gbc);
		add(DescriptionArea);
		gbc.weighty = 0.0;

		// put the space back
		gbc.insets.top = gbc.insets.bottom = 2;

		gbc.gridwidth = 1;
		lbl = new Label("Author:", Label.RIGHT);
		layout.setConstraints(lbl, gbc);
		add(lbl);

		gbc.gridwidth = 2;				// give choices a little extra
		layout.setConstraints(RequestorChoice, gbc);
		add(RequestorChoice);

		gbc.gridwidth = 1;
		lbl = new Label("Budget:", Label.RIGHT);
		layout.setConstraints(lbl, gbc);
		add(lbl);

		gbc.gridwidth = 2;				// give choices a little extra
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		layout.setConstraints(ApprovedBudgetChoice, gbc);
		add(ApprovedBudgetChoice);

		gbc.gridwidth = 1;
		lbl = new Label("Status:", Label.RIGHT);
		layout.setConstraints(lbl, gbc);
		add(lbl);

		// add the choices before adding the control
		StatusChoice.addItemData("Submitted", Request.SUBMITTED);
		StatusChoice.addItemData("Approved", Request.APPROVED);
		StatusChoice.addItemData("Completed", Request.COMPLETED);
		StatusChoice.addItemData("Postponed", Request.POSTPONED);

		gbc.gridwidth = 2;				// give choices a little extra
		layout.setConstraints(StatusChoice, gbc);
		add(StatusChoice);

		gbc.gridwidth = 1;
		lbl = new Label("Completed:", Label.RIGHT);
		layout.setConstraints(lbl, gbc);
		add(lbl);

		gbc.gridwidth = GridBagConstraints.REMAINDER;
		CompletionDateField.setEditable(false);
		layout.setConstraints(CompletionDateField, gbc);
		add(CompletionDateField);

		gbc.fill = GridBagConstraints.NONE;
		Button btn = new Button("Save");
		layout.setConstraints(btn, gbc);
		add(btn);

		// load choice lists
		loadOperators();
		loadDepartments();
	}

	public boolean action(Event evt, Object obj)
	{
		boolean retval = false;			// assume no action

		if ("Save".equals(obj))
		{
			if (validateFields())
			{
				try
				{
					CurrentRequest.write(db);

					conPanel.readRequests();
				}
				catch(ComException e)
				{
					setStatus(e);
				}
			}

			retval = true;
		}

		return retval;
	}

	public void setRequest(Request req)
	{
		// save the request as the current request
		CurrentRequest = req;

		TrackingIdField.setText(CurrentRequest.getTrackingIdStr());
		SubmissionDateField.setText(CurrentRequest.SubmissionDate.toLocaleString());
		CompletionDateField.setText(CurrentRequest.getCompletionDate());
		TitleField.setText(CurrentRequest.Title);
		DescriptionArea.setText(CurrentRequest.Description);
		RequestorChoice.select(CurrentRequest.Requestor.toString());
		StatusChoice.selectByData(CurrentRequest.Status);
		ApprovedBudgetChoice.select(CurrentRequest.ApprovedBudget.toString());
	}

	private void loadOperators()
	{
		Vector operators = new Vector();

		Operator.readAll(db, operators);

		RequestorChoice.load(operators);
	}

	private void loadDepartments()
	{
		Vector depts = new Vector();

		Department.readAll(db, depts);

		ApprovedBudgetChoice.load(depts);
	}

	private boolean validateFields()
	{
		boolean retval = false;			// assume not enough info

		if (TitleField.getText().length() > 0)
		{
			CurrentRequest.Title = TitleField.getText();
			CurrentRequest.Description = DescriptionArea.getText();
			CurrentRequest.Requestor = (Operator)RequestorChoice.getSelectedData();
			CurrentRequest.Status = (String)StatusChoice.getSelectedData();
			CurrentRequest.ApprovedBudget = (Department)ApprovedBudgetChoice.getSelectedData();

			retval = true;
		}

		return retval;
	}
}
