import java.util.Date;
import java.util.Vector;
import dao_dbengine;
import dao3032.*;
import com.ms.com.Variant;

/*
 *
 * Request
 *
 * This class takes care of reading and writing a request
 *   to the database.
 *
 */
public class Request 
{
	// constants used in the class
	public final static Date DATE_NOT_SET = new Date(0);
	public final static int NEW_REQUEST_ID = -1;

	// these are the constants that should be in ACCESS_LEVEL
	public final static String SUBMITTED = "S";
	public final static String APPROVED = "A";
	public final static String COMPLETED = "C";
	public final static String POSTPONED = "P";

	public int TrackingId;
	public Operator Requestor = new Operator();
	public String Title;
	public String Description;
	public Date SubmissionDate = new Date();
	public Date CompletionDate = new Date();
	public String Status;
	public Department ApprovedBudget = new Department();

	public Request()
	{
		clear();
	}

	public Request(int TrackingId, String Title)
	{
		clear();

		this.TrackingId = TrackingId;
		this.Title = Title;
	}

	public String getTrackingIdStr()
	{
		return TrackingId == NEW_REQUEST_ID ? "<new>" : 
				Integer.toString(TrackingId);
	}

	public String getCompletionDate()
	{
		return CompletionDate == DATE_NOT_SET ? "" : 
				CompletionDate.toLocaleString();
	}

	public String toString()
	{
		return getTrackingIdStr() + ": " + Title;
	}

	public void clear()
	{
        TrackingId = NEW_REQUEST_ID;
        Requestor.clear();
		Title = Description = "";
		Status = SUBMITTED;
		SubmissionDate = new Date();	// set to now
		CompletionDate = DATE_NOT_SET;
		ApprovedBudget.clear();
	}

	public void set(Request r)
	{
		TrackingId = r.TrackingId;
		Requestor.set(r.Requestor);
		Title = r.Title;
		Status = r.Status;
		Description = r.Description;
		SubmissionDate = r.SubmissionDate;
		CompletionDate = r.CompletionDate;
		ApprovedBudget.set(r.ApprovedBudget);
	}

	public boolean read(Database db)
	{
		boolean retval = false;			// assume database error
		String command = "select * from REQUEST where TRACKING_ID = " + 
				Integer.toString(TrackingId);
		Variant type = new Variant();
		Variant options = new Variant();
		Variant fieldName = new Variant();
		Variant fieldValue = new Variant();

		type.putShort(Constants.dbOpenDynaset);
		options.putShort(Constants.dbReadOnly);

		// create the recordset and get the number of records returned
		Recordset recordset = db.OpenRecordset(command, type, options);

		int recordCount = recordset.getRecordCount();

		// retrieve the first row
		if (recordCount > 0) {
			recordset.MoveFirst();

			// get the fields of the row
			Fields fields = recordset.getFields();
			_Field field;

			// retrieve all of the information and store it
			fieldName.putString("TITLE");
			field = fields.getItem(fieldName);
			fieldValue = field.getValue();
			Title = fieldValue.toString();

			fieldName.putString("DESCRIPTION");
			field = fields.getItem(fieldName);
			fieldValue = field.getValue();
			Description = fieldValue.toString();

			fieldName.putString("REQUESTOR");
			field = fields.getItem(fieldName);
			fieldValue = field.getValue();
			Requestor.setUserName(fieldValue.toString());

			fieldName.putString("SUBMISSION_DATE");
			field = fields.getItem(fieldName);
			fieldValue = field.getValue();
			SubmissionDate = new Date(fieldValue.toString());

			fieldName.putString("COMPLETION_DATE");
			field = fields.getItem(fieldName);
			fieldValue = field.getValue();
			if (fieldValue.getvt() != Variant.VariantNull)
				CompletionDate = new Date(fieldValue.toString());

			fieldName.putString("STATUS");
			field = fields.getItem(fieldName);
			fieldValue = field.getValue();
			Status = fieldValue.toString();

			fieldName.putString("APPROVED_BUDGET");
			field = fields.getItem(fieldName);
			fieldValue = field.getValue();
			ApprovedBudget.setCode(fieldValue.toString());

			retval = true;
		}

		if (retval)
		{
			retval = Requestor.read(db);
			if (retval)
				retval = ApprovedBudget.read(db);
		}

		return retval;
	}

	public void write(Database db)
	{
		StringBuffer command = new StringBuffer();

		// set completion date if needed 
		if (Status.equals(COMPLETED) && CompletionDate.equals(DATE_NOT_SET))
			CompletionDate = new Date();

		if (TrackingId == NEW_REQUEST_ID)
		{
			command.append("insert into REQUEST(TITLE,DESCRIPTION,");
			command.append("REQUESTOR,SUBMISSION_DATE,COMPLETION_DATE,");
			command.append("STATUS,APPROVED_BUDGET) values(");
			command.append("'" + Title + "',");
			command.append("'" + Description + "',");
			command.append("'" + Requestor.UserName + "',");
			command.append("'" + SubmissionDate.toLocaleString() + "',");
			command.append((CompletionDate != DATE_NOT_SET) ? 
					"'" + CompletionDate.toLocaleString() + "'," : "NULL,");
			command.append((Status != null) ? "'" + Status + "'," : "NULL,");
			command.append("'" + ApprovedBudget.Code + "')");
		}
		else
		{
			command.append("update REQUEST set ");
			command.append("TITLE = '" + Title + "',");
			command.append("DESCRIPTION = '" + Description + "',");
			command.append("REQUESTOR = '" + Requestor.UserName + "',");
			command.append("SUBMISSION_DATE = '" + SubmissionDate.toLocaleString() + "',");
			command.append("COMPLETION_DATE = ");
			command.append((CompletionDate != DATE_NOT_SET) ? 
					"'" + CompletionDate.toLocaleString() + "'," : "NULL,");
			command.append("STATUS = ");
			command.append((Status != null) ? "'" + Status + "'," : "NULL,");
			command.append("APPROVED_BUDGET = '" + ApprovedBudget.Code + "' ");
			command.append("where TRACKING_ID = " + Integer.toString(TrackingId));
		}

		Variant options = new Variant();
		options.putShort(Constants.dbFailOnError);

		db.Execute(command.toString(), options);
	}

	public static boolean readAll(Database db, Vector requestList)
	{
		boolean retval = false;			// assume database error
		String command = "select TRACKING_ID, TITLE from REQUEST order by TRACKING_ID";
		Variant type = new Variant();
		Variant options = new Variant();
		Variant fieldName = new Variant();
		Variant fieldValue = new Variant();

		type.putShort(Constants.dbOpenDynaset);
		options.putShort(Constants.dbReadOnly);

		// create the recordset and get the number of records returned
		Recordset recordset = db.OpenRecordset(command, type, options);

		int retrievedRecordCount = 0;

		// retrieve the first row
		if (recordset.getRecordCount() > 0)
			recordset.MoveFirst();

		// read all of the records
		while (retrievedRecordCount < recordset.getRecordCount())
		{
			int trackingId;
			String title;

			// get the fields of the row
			Fields fields = recordset.getFields();
			_Field field;

			// retrieve all of the information and store it
			fieldName.putString("TRACKING_ID");
			field = fields.getItem(fieldName);
			fieldValue = field.getValue();
			trackingId = fieldValue.toInt();

			fieldName.putString("TITLE");
			field = fields.getItem(fieldName);
			fieldValue = field.getValue();
			title = fieldValue.toString();

			// save the request
			requestList.addElement(new Request(trackingId, title));

			// move the element indexing
			recordset.MoveNext();
			retrievedRecordCount++;
		}

		return retval;
	}
}
