import java.util.*;

/*
 * UserInformation
 *
 * This class contains information about the current user.
 */
public class UserInformation extends Observable 
{
	// the following are used to indicate the access level
	public final static int ADMINISTRATOR = 0;	// database level
	public final static int EXECUTIVE = 1;
	public final static int MANAGER = 2;
	public final static int PROGRAMMER = 3;
	public final static int GUEST = 4;

	// the following are used to name the possible screens
	public final static String LOGIN = "Login";
	public final static String CONTROL = "Control";
	public final static String ADMINISTRATION = "Administration";

	public String UserName;
	public String FirstName;
	public String LastName;
	public int AccessLevel;
	public String NextScreen = LOGIN;

	// This method is used to convert database text to an 
	//   interger access level.
	public void setAccessLevel(String AccessLevelStr)
	{
		switch (AccessLevelStr.charAt(0))
		{
		case 'A':
			AccessLevel = ADMINISTRATOR;
			break;

		case 'E':
			AccessLevel = EXECUTIVE;
			break;

		case 'M':
			AccessLevel = MANAGER;
			break;

		case 'P':
			AccessLevel = PROGRAMMER;
			break;

		default:
			AccessLevel = GUEST;
			break;
		}
	}

	// This method controls the order that the screens are displayed.
	public void signalNextScreen()
	{
		// if at the login or administration screens, then go back
		//   to the control; otherwise, go to the administration
		if ((NextScreen == LOGIN) || (NextScreen == ADMINISTRATION))
			NextScreen = CONTROL;
		else
			NextScreen = ADMINISTRATION;

		// notify on lookers for actual changing of the panels
		done();
	}

	// This method is used to signal all oberservers that 
	//   this class is done changing.
	private void done()
	{
		setChanged();
		notifyObservers();
	}
}
